summaryrefslogtreecommitdiff
path: root/shared-bindings/pulseio
diff options
context:
space:
mode:
Diffstat (limited to 'shared-bindings/pulseio')
-rw-r--r--shared-bindings/pulseio/PWMOut.c245
-rw-r--r--shared-bindings/pulseio/PWMOut.h58
-rw-r--r--shared-bindings/pulseio/PulseOut.c39
-rw-r--r--shared-bindings/pulseio/PulseOut.h8
-rw-r--r--shared-bindings/pulseio/__init__.c31
5 files changed, 40 insertions, 341 deletions
diff --git a/shared-bindings/pulseio/PWMOut.c b/shared-bindings/pulseio/PWMOut.c
deleted file mode 100644
index e9b6c45d2..000000000
--- a/shared-bindings/pulseio/PWMOut.c
+++ /dev/null
@@ -1,245 +0,0 @@
-/*
- * This file is part of the Micro Python project, http://micropython.org/
- *
- * The MIT License (MIT)
- *
- * SPDX-FileCopyrightText: Copyright (c) 2016 Damien P. George
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
- */
-
-#include <stdint.h>
-
-#include "lib/utils/context_manager_helpers.h"
-#include "py/objproperty.h"
-#include "py/runtime.h"
-
-#include "shared-bindings/microcontroller/Pin.h"
-#include "shared-bindings/pulseio/PWMOut.h"
-#include "shared-bindings/util.h"
-#include "supervisor/shared/translate.h"
-
-//| class PWMOut:
-//| """Output a Pulse Width Modulated signal on a given pin."""
-//|
-//| def __init__(self, pin: microcontroller.Pin, *, duty_cycle: int = 0, frequency: int = 500, variable_frequency: bool = False) -> None:
-//| """Create a PWM object associated with the given pin. This allows you to
-//| write PWM signals out on the given pin. Frequency is fixed after init
-//| unless ``variable_frequency`` is True.
-//|
-//| .. note:: When ``variable_frequency`` is True, further PWM outputs may be
-//| limited because it may take more internal resources to be flexible. So,
-//| when outputting both fixed and flexible frequency signals construct the
-//| fixed outputs first.
-//|
-//| :param ~microcontroller.Pin pin: The pin to output to
-//| :param int duty_cycle: The fraction of each pulse which is high. 16-bit
-//| :param int frequency: The target frequency in Hertz (32-bit)
-//| :param bool variable_frequency: True if the frequency will change over time
-//|
-//| Simple LED fade::
-//|
-//| import pulseio
-//| import board
-//|
-//| pwm = pulseio.PWMOut(board.D13) # output on D13
-//| pwm.duty_cycle = 2 ** 15 # Cycles the pin with 50% duty cycle (half of 2 ** 16) at the default 500hz
-//|
-//| PWM at specific frequency (servos and motors)::
-//|
-//| import pulseio
-//| import board
-//|
-//| pwm = pulseio.PWMOut(board.D13, frequency=50)
-//| pwm.duty_cycle = 2 ** 15 # Cycles the pin with 50% duty cycle (half of 2 ** 16) at 50hz
-//|
-//| Variable frequency (usually tones)::
-//|
-//| import pulseio
-//| import board
-//| import time
-//|
-//| pwm = pulseio.PWMOut(board.D13, duty_cycle=2 ** 15, frequency=440, variable_frequency=True)
-//| time.sleep(0.2)
-//| pwm.frequency = 880
-//| time.sleep(0.1)"""
-//| ...
-//|
-STATIC mp_obj_t pulseio_pwmout_make_new(const mp_obj_type_t *type, size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
- enum { ARG_pin, ARG_duty_cycle, ARG_frequency, ARG_variable_frequency };
- static const mp_arg_t allowed_args[] = {
- { MP_QSTR_pin, MP_ARG_REQUIRED | MP_ARG_OBJ, },
- { MP_QSTR_duty_cycle, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = 0} },
- { MP_QSTR_frequency, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = 500} },
- { MP_QSTR_variable_frequency, MP_ARG_KW_ONLY | MP_ARG_BOOL, {.u_bool = false} },
- };
- mp_arg_val_t parsed_args[MP_ARRAY_SIZE(allowed_args)];
- mp_arg_parse_all(n_args, args, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, parsed_args);
-
- const mcu_pin_obj_t *pin = validate_obj_is_free_pin(parsed_args[ARG_pin].u_obj);
-
- uint16_t duty_cycle = parsed_args[ARG_duty_cycle].u_int;
- uint32_t frequency = parsed_args[ARG_frequency].u_int;
- bool variable_frequency = parsed_args[ARG_variable_frequency].u_bool;
-
- // create PWM object from the given pin
- pulseio_pwmout_obj_t *self = m_new_obj(pulseio_pwmout_obj_t);
- self->base.type = &pulseio_pwmout_type;
- pwmout_result_t result = common_hal_pulseio_pwmout_construct(self, pin, duty_cycle, frequency, variable_frequency);
- if (result == PWMOUT_INVALID_PIN) {
- mp_raise_ValueError(translate("Invalid pin"));
- } else if (result == PWMOUT_INVALID_FREQUENCY) {
- mp_raise_ValueError(translate("Invalid PWM frequency"));
- } else if (result == PWMOUT_ALL_TIMERS_ON_PIN_IN_USE) {
- mp_raise_ValueError(translate("All timers for this pin are in use"));
- } else if (result == PWMOUT_ALL_TIMERS_IN_USE) {
- mp_raise_RuntimeError(translate("All timers in use"));
- }
-
- return MP_OBJ_FROM_PTR(self);
-}
-
-//| def deinit(self) -> None:
-//| """Deinitialises the PWMOut and releases any hardware resources for reuse."""
-//| ...
-//|
-STATIC mp_obj_t pulseio_pwmout_deinit(mp_obj_t self_in) {
- pulseio_pwmout_obj_t *self = MP_OBJ_TO_PTR(self_in);
- common_hal_pulseio_pwmout_deinit(self);
- return mp_const_none;
-}
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(pulseio_pwmout_deinit_obj, pulseio_pwmout_deinit);
-
-STATIC void check_for_deinit(pulseio_pwmout_obj_t *self) {
- if (common_hal_pulseio_pwmout_deinited(self)) {
- raise_deinited_error();
- }
-}
-
-//| def __enter__(self) -> PWMOut:
-//| """No-op used by Context Managers."""
-//| ...
-//|
-// Provided by context manager helper.
-
-//| def __exit__(self) -> None:
-//| """Automatically deinitializes the hardware when exiting a context. See
-//| :ref:`lifetime-and-contextmanagers` for more info."""
-//| ...
-//|
-STATIC mp_obj_t pulseio_pwmout_obj___exit__(size_t n_args, const mp_obj_t *args) {
- (void)n_args;
- common_hal_pulseio_pwmout_deinit(args[0]);
- return mp_const_none;
-}
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pulseio_pwmout___exit___obj, 4, 4, pulseio_pwmout_obj___exit__);
-
-//| duty_cycle: int
-//| """16 bit value that dictates how much of one cycle is high (1) versus low
-//| (0). 0xffff will always be high, 0 will always be low and 0x7fff will
-//| be half high and then half low.
-//|
-//| Depending on how PWM is implemented on a specific board, the internal
-//| representation for duty cycle might have less than 16 bits of resolution.
-//| Reading this property will return the value from the internal representation,
-//| so it may differ from the value set."""
-//|
-STATIC mp_obj_t pulseio_pwmout_obj_get_duty_cycle(mp_obj_t self_in) {
- pulseio_pwmout_obj_t *self = MP_OBJ_TO_PTR(self_in);
- check_for_deinit(self);
- return MP_OBJ_NEW_SMALL_INT(common_hal_pulseio_pwmout_get_duty_cycle(self));
-}
-MP_DEFINE_CONST_FUN_OBJ_1(pulseio_pwmout_get_duty_cycle_obj, pulseio_pwmout_obj_get_duty_cycle);
-
-STATIC mp_obj_t pulseio_pwmout_obj_set_duty_cycle(mp_obj_t self_in, mp_obj_t duty_cycle) {
- pulseio_pwmout_obj_t *self = MP_OBJ_TO_PTR(self_in);
- check_for_deinit(self);
- mp_int_t duty = mp_obj_get_int(duty_cycle);
- if (duty < 0 || duty > 0xffff) {
- mp_raise_ValueError(translate("PWM duty_cycle must be between 0 and 65535 inclusive (16 bit resolution)"));
- }
- common_hal_pulseio_pwmout_set_duty_cycle(self, duty);
- return mp_const_none;
-}
-MP_DEFINE_CONST_FUN_OBJ_2(pulseio_pwmout_set_duty_cycle_obj, pulseio_pwmout_obj_set_duty_cycle);
-
-const mp_obj_property_t pulseio_pwmout_duty_cycle_obj = {
- .base.type = &mp_type_property,
- .proxy = {(mp_obj_t)&pulseio_pwmout_get_duty_cycle_obj,
- (mp_obj_t)&pulseio_pwmout_set_duty_cycle_obj,
- (mp_obj_t)&mp_const_none_obj},
-};
-
-//| frequency: int
-//| """32 bit value that dictates the PWM frequency in Hertz (cycles per
-//| second). Only writeable when constructed with ``variable_frequency=True``.
-//|
-//| Depending on how PWM is implemented on a specific board, the internal value
-//| for the PWM's duty cycle may need to be recalculated when the frequency
-//| changes. In these cases, the duty cycle is automatically recalculated
-//| from the original duty cycle value. This should happen without any need
-//| to manually re-set the duty cycle."""
-//|
-STATIC mp_obj_t pulseio_pwmout_obj_get_frequency(mp_obj_t self_in) {
- pulseio_pwmout_obj_t *self = MP_OBJ_TO_PTR(self_in);
- check_for_deinit(self);
- return MP_OBJ_NEW_SMALL_INT(common_hal_pulseio_pwmout_get_frequency(self));
-}
-MP_DEFINE_CONST_FUN_OBJ_1(pulseio_pwmout_get_frequency_obj, pulseio_pwmout_obj_get_frequency);
-
-STATIC mp_obj_t pulseio_pwmout_obj_set_frequency(mp_obj_t self_in, mp_obj_t frequency) {
- pulseio_pwmout_obj_t *self = MP_OBJ_TO_PTR(self_in);
- check_for_deinit(self);
- if (!common_hal_pulseio_pwmout_get_variable_frequency(self)) {
- mp_raise_AttributeError(translate(
- "PWM frequency not writable when variable_frequency is False on "
- "construction."));
- }
- common_hal_pulseio_pwmout_set_frequency(self, mp_obj_get_int(frequency));
- return mp_const_none;
-}
-MP_DEFINE_CONST_FUN_OBJ_2(pulseio_pwmout_set_frequency_obj, pulseio_pwmout_obj_set_frequency);
-
-const mp_obj_property_t pulseio_pwmout_frequency_obj = {
- .base.type = &mp_type_property,
- .proxy = {(mp_obj_t)&pulseio_pwmout_get_frequency_obj,
- (mp_obj_t)&pulseio_pwmout_set_frequency_obj,
- (mp_obj_t)&mp_const_none_obj},
-};
-
-STATIC const mp_rom_map_elem_t pulseio_pwmout_locals_dict_table[] = {
- // Methods
- { MP_ROM_QSTR(MP_QSTR_deinit), MP_ROM_PTR(&pulseio_pwmout_deinit_obj) },
- { MP_ROM_QSTR(MP_QSTR___enter__), MP_ROM_PTR(&default___enter___obj) },
- { MP_ROM_QSTR(MP_QSTR___exit__), MP_ROM_PTR(&pulseio_pwmout___exit___obj) },
-
- // Properties
- { MP_ROM_QSTR(MP_QSTR_duty_cycle), MP_ROM_PTR(&pulseio_pwmout_duty_cycle_obj) },
- { MP_ROM_QSTR(MP_QSTR_frequency), MP_ROM_PTR(&pulseio_pwmout_frequency_obj) },
- // TODO(tannewt): Add enabled to determine whether the signal is output
- // without giving up the resources. Useful for IR output.
-};
-STATIC MP_DEFINE_CONST_DICT(pulseio_pwmout_locals_dict, pulseio_pwmout_locals_dict_table);
-
-const mp_obj_type_t pulseio_pwmout_type = {
- { &mp_type_type },
- .name = MP_QSTR_PWMOut,
- .make_new = pulseio_pwmout_make_new,
- .locals_dict = (mp_obj_dict_t*)&pulseio_pwmout_locals_dict,
-};
diff --git a/shared-bindings/pulseio/PWMOut.h b/shared-bindings/pulseio/PWMOut.h
deleted file mode 100644
index c72278e0e..000000000
--- a/shared-bindings/pulseio/PWMOut.h
+++ /dev/null
@@ -1,58 +0,0 @@
-/*
- * This file is part of the Micro Python project, http://micropython.org/
- *
- * The MIT License (MIT)
- *
- * SPDX-FileCopyrightText: Copyright (c) 2013, 2014 Damien P. George
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
- */
-
-#ifndef MICROPY_INCLUDED_SHARED_BINDINGS_PULSEIO_PWMOUT_H
-#define MICROPY_INCLUDED_SHARED_BINDINGS_PULSEIO_PWMOUT_H
-
-#include "common-hal/microcontroller/Pin.h"
-#include "common-hal/pulseio/PWMOut.h"
-
-extern const mp_obj_type_t pulseio_pwmout_type;
-
-typedef enum {
- PWMOUT_OK,
- PWMOUT_INVALID_PIN,
- PWMOUT_INVALID_FREQUENCY,
- PWMOUT_ALL_TIMERS_ON_PIN_IN_USE,
- PWMOUT_ALL_TIMERS_IN_USE
-} pwmout_result_t;
-
-extern pwmout_result_t common_hal_pulseio_pwmout_construct(pulseio_pwmout_obj_t* self,
- const mcu_pin_obj_t* pin, uint16_t duty, uint32_t frequency,
- bool variable_frequency);
-extern void common_hal_pulseio_pwmout_deinit(pulseio_pwmout_obj_t* self);
-extern bool common_hal_pulseio_pwmout_deinited(pulseio_pwmout_obj_t* self);
-extern void common_hal_pulseio_pwmout_set_duty_cycle(pulseio_pwmout_obj_t* self, uint16_t duty);
-extern uint16_t common_hal_pulseio_pwmout_get_duty_cycle(pulseio_pwmout_obj_t* self);
-extern void common_hal_pulseio_pwmout_set_frequency(pulseio_pwmout_obj_t* self, uint32_t frequency);
-extern uint32_t common_hal_pulseio_pwmout_get_frequency(pulseio_pwmout_obj_t* self);
-extern bool common_hal_pulseio_pwmout_get_variable_frequency(pulseio_pwmout_obj_t* self);
-
-// This is used by the supervisor to claim PWMOut devices indefinitely.
-extern void common_hal_pulseio_pwmout_never_reset(pulseio_pwmout_obj_t *self);
-extern void common_hal_pulseio_pwmout_reset_ok(pulseio_pwmout_obj_t *self);
-
-#endif // MICROPY_INCLUDED_SHARED_BINDINGS_PULSEIO_PWMOUT_H
diff --git a/shared-bindings/pulseio/PulseOut.h b/shared-bindings/pulseio/PulseOut.h
index 390910ff6..49dc555fe 100644
--- a/shared-bindings/pulseio/PulseOut.h
+++ b/shared-bindings/pulseio/PulseOut.h
@@ -29,12 +29,16 @@
#include "common-hal/microcontroller/Pin.h"
#include "common-hal/pulseio/PulseOut.h"
-#include "common-hal/pulseio/PWMOut.h"
+#include "common-hal/pwmio/PWMOut.h"
extern const mp_obj_type_t pulseio_pulseout_type;
extern void common_hal_pulseio_pulseout_construct(pulseio_pulseout_obj_t* self,
- const pulseio_pwmout_obj_t* carrier);
+ const pwmio_pwmout_obj_t* carrier,
+ const mcu_pin_obj_t* pin,
+ uint32_t frequency,
+ uint16_t duty_cycle);
+
extern void common_hal_pulseio_pulseout_deinit(pulseio_pulseout_obj_t* self);
extern bool common_hal_pulseio_pulseout_deinited(pulseio_pulseout_obj_t* self);
extern void common_hal_pulseio_pulseout_send(pulseio_pulseout_obj_t* self,
diff --git a/shared-bindings/pulseio/__init__.c b/shared-bindings/pulseio/__init__.c
index 87946e5f0..bfe9635f0 100644
--- a/shared-bindings/pulseio/__init__.c
+++ b/shared-bindings/pulseio/__init__.c
@@ -33,40 +33,29 @@
#include "shared-bindings/pulseio/__init__.h"
#include "shared-bindings/pulseio/PulseIn.h"
#include "shared-bindings/pulseio/PulseOut.h"
-#include "shared-bindings/pulseio/PWMOut.h"
+#include "shared-bindings/pwmio/PWMOut.h"
-//| """Support for pulse based protocols
+//| """Support for individual pulse based protocols
//|
//| The `pulseio` module contains classes to provide access to basic pulse IO.
+//| Individual pulses are commonly used in infrared remotes and in DHT
+//| temperature sensors.
+//|
+//|
+//| .. warning:: PWMOut is moving to `pwmio` and will be removed from `pulseio`
+//| in CircuitPython 7.
//|
-
//| All classes change hardware state and should be deinitialized when they
//| are no longer needed if the program continues after use. To do so, either
//| call :py:meth:`!deinit` or use a context manager. See
-//| :ref:`lifetime-and-contextmanagers` for more info.
-//|
-//| For example::
-//|
-//| import pulseio
-//| import time
-//| from board import *
-//|
-//| pwm = pulseio.PWMOut(D13)
-//| pwm.duty_cycle = 2 ** 15
-//| time.sleep(0.1)
-//|
-//| This example will initialize the the device, set
-//| :py:data:`~pulseio.PWMOut.duty_cycle`, and then sleep 0.1 seconds.
-//| CircuitPython will automatically turn off the PWM when it resets all
-//| hardware after program completion. Use ``deinit()`` or a ``with`` statement
-//| to do it yourself."""
+//| :ref:`lifetime-and-contextmanagers` for more info."""
//|
STATIC const mp_rom_map_elem_t pulseio_module_globals_table[] = {
{ MP_ROM_QSTR(MP_QSTR___name__), MP_ROM_QSTR(MP_QSTR_pulseio) },
{ MP_ROM_QSTR(MP_QSTR_PulseIn), MP_ROM_PTR(&pulseio_pulsein_type) },
{ MP_ROM_QSTR(MP_QSTR_PulseOut), MP_ROM_PTR(&pulseio_pulseout_type) },
- { MP_ROM_QSTR(MP_QSTR_PWMOut), MP_ROM_PTR(&pulseio_pwmout_type) },
+ { MP_ROM_QSTR(MP_QSTR_PWMOut), MP_ROM_PTR(&pwmio_pwmout_type) },
};
STATIC MP_DEFINE_CONST_DICT(pulseio_module_globals, pulseio_module_globals_table);
diff --git a/shared-bindings/pulseio/PulseOut.c b/shared-bindings/pulseio/PulseOut.c
index 3d35f0544..2b787df4f 100644
--- a/shared-bindings/pulseio/PulseOut.c
+++ b/shared-bindings/pulseio/PulseOut.c
@@ -32,7 +32,7 @@
#include "shared-bindings/microcontroller/Pin.h"
#include "shared-bindings/pulseio/PulseOut.h"
-#include "shared-bindings/pulseio/PWMOut.h"
+#include "shared-bindings/pwmio/PWMOut.h"
#include "shared-bindings/util.h"
#include "supervisor/shared/translate.h"
@@ -41,19 +41,20 @@
//| pulsed signal consists of timed on and off periods. Unlike PWM, there is no set duration
//| for on and off pairs."""
//|
//| def __init__(self, carrier: PWMOut) -> None:
//| """Create a PulseOut object associated with the given PWMout object.
//|
//| :param ~pulseio.PWMOut carrier: PWMOut that is set to output on the desired pin.
//|
//| Send a short series of pulses::
//|
//| import array
//| import pulseio
+//| import pwmio
//| import board
//|
//| # 50% duty cycle at 38kHz.
-//| pwm = pulseio.PWMOut(board.D13, frequency=38000, duty_cycle=32768)
+//| pwm = pwmio.PWMOut(board.D13, frequency=38000, duty_cycle=32768)
//| pulse = pulseio.PulseOut(pwm)
//| # on off on off on
//| pulses = array.array('H', [65000, 1000, 65000, 65000, 1000])
@@ -64,20 +65,28 @@
//| pulse.send(pulses)"""
//| ...
//|
-STATIC mp_obj_t pulseio_pulseout_make_new(const mp_obj_type_t *type, size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
- mp_arg_check_num(n_args, kw_args, 1, 1, false);
- mp_obj_t carrier_obj = args[0];
-
- if (!MP_OBJ_IS_TYPE(carrier_obj, &pulseio_pwmout_type)) {
- mp_raise_TypeError_varg(translate("Expected a %q"), pulseio_pwmout_type.name);
- }
-
- // create Pulse object from the given pin
+STATIC mp_obj_t pulseio_pulseout_make_new(const mp_obj_type_t *type, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
pulseio_pulseout_obj_t *self = m_new_obj(pulseio_pulseout_obj_t);
self->base.type = &pulseio_pulseout_type;
- common_hal_pulseio_pulseout_construct(self, (pulseio_pwmout_obj_t *)MP_OBJ_TO_PTR(carrier_obj));
-
+ mp_obj_t carrier_obj = pos_args[0];
+ if (MP_OBJ_IS_TYPE(carrier_obj, &pwmio_pwmout_type)) {
+ // Use a PWMOut Carrier
+ mp_arg_check_num(n_args, kw_args, 1, 1, false);
+ common_hal_pulseio_pulseout_construct(self, (pwmio_pwmout_obj_t *)MP_OBJ_TO_PTR(carrier_obj), NULL, 0, 0);
+ } else {
+ // Use a Pin, frequency, and duty cycle
+ enum { ARG_pin, ARG_frequency};
+ static const mp_arg_t allowed_args[] = {
+ { MP_QSTR_pin, MP_ARG_REQUIRED | MP_ARG_OBJ },
+ { MP_QSTR_frequency, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = 38000} },
+ { MP_QSTR_duty_cycle, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = 1<<15} },
+ };
+ mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
+ mp_arg_parse_all(n_args, pos_args, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
+ const mcu_pin_obj_t* pin = validate_obj_is_free_pin(args[ARG_pin].u_obj);
+ common_hal_pulseio_pulseout_construct(self, NULL, pin, args[ARG_frequency].u_int, args[ARG_frequency].u_int);
+ }
return MP_OBJ_FROM_PTR(self);
}
@@ -41,19 +41,20 @@
//| for on and off pairs."""
//|
-//| def __init__(self, carrier: PWMOut) -> None:
+//| def __init__(self, carrier: pwmio.PWMOut) -> None:
//| """Create a PulseOut object associated with the given PWMout object.
//|
@@ -41,19 +41,20 @@
//| """Create a PulseOut object associated with the given PWMout object.
//|
-//| :param ~pulseio.PWMOut carrier: PWMOut that is set to output on the desired pin.
+//| :param ~pwmio.PWMOut carrier: PWMOut that is set to output on the desired pin.
//|
//| Send a short series of pulses::